/* Copyright (c) 2006, Catalin Patulea <cat@vv.carleton.ca> All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 * 
 *     * Neither the name of Catalin Patulea nor the names of its contributors may 
 *     be used to endorse or promote products derived from this software without 
 *     specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "I2CMaster.h"
#include "Display.h"
#include "Util.h"
#include "resource.h"

namespace I2CMaster {

// Macro to compute absolute address from register offset, to facilitate
// rebasing the peripheral.
#define I2C_MASTER_ADDR(o) (I2C_MASTER_BASE + (o))

// From osram96x16.c
#define SSD0303_ADDR            0x3d

BOOL DefineAllSFR() {
	// TODO: Define bits using DefineSFR and non-zero bBitPos.
	return
	Agsi::DefineSFR("I2CMSA",	I2C_MASTER_ADDR(I2C_MASTER_O_SA),	AGSILONG, 0) &&
	Agsi::DefineSFR("I2CMCS",	I2C_MASTER_ADDR(I2C_MASTER_O_CS),	AGSILONG, 0) &&
	Agsi::DefineSFR("I2CMDR",	I2C_MASTER_ADDR(I2C_MASTER_O_DR),	AGSILONG, 0) &&
	Agsi::DefineSFR("I2CMTPR",	I2C_MASTER_ADDR(I2C_MASTER_O_TPR),	AGSILONG, 0) &&
	Agsi::DefineSFR("I2CMIMR",	I2C_MASTER_ADDR(I2C_MASTER_O_IMR),	AGSILONG, 0) &&
	Agsi::DefineSFR("I2CMRIS",	I2C_MASTER_ADDR(I2C_MASTER_O_RIS),	AGSILONG, 0) &&
	Agsi::DefineSFR("I2CMMIS",	I2C_MASTER_ADDR(I2C_MASTER_O_MIS),	AGSILONG, 0) &&
	Agsi::DefineSFR("I2CMICR",	I2C_MASTER_ADDR(I2C_MASTER_O_MICR),	AGSILONG, 0) &&
	Agsi::DefineSFR("I2CMCR",	I2C_MASTER_ADDR(I2C_MASTER_O_CR),	AGSILONG, 0);
}

BOOL DefineAllVTREG() {
	return TRUE;
}

BOOL DefineAllInterrupts() {
	// TODO: Implement interrupt.
	return TRUE;
}

#define TRACE_UPDATE(r) do { \
	Agsi::ReadSFR(I2C_MASTER_ADDR(I2C_MASTER_O_##r), &Current, &Previous, (DWORD)-1); \
	if (Previous != Current) { \
		trace("%s: 0x%08x --> 0x%08x\n", #r, Previous, Current); \
	} \
} while (0)

std::deque<unsigned char> Queue;

BOOL InWriteSFR = FALSE;

static void UpdateCS(void) {
	DWORD Previous, Current;
#if 0
	TRACE_UPDATE(SA);
	TRACE_UPDATE(CS);
	TRACE_UPDATE(DR);
	TRACE_UPDATE(TPR);
	TRACE_UPDATE(IMR);
	TRACE_UPDATE(RIS);
	TRACE_UPDATE(MIS);
	TRACE_UPDATE(MICR);
	TRACE_UPDATE(CR);
#endif

	if (InWriteSFR) {
		// Don't cause infinite loop when updating ourselves
		return;
	}

	// Receive or send
	DWORD RS, PreviousRS;
	Agsi::ReadSFR(I2C_MASTER_ADDR(I2C_MASTER_O_SA), &RS, &PreviousRS, I2C_MASTER_SA_RS);

	// Slave address
	DWORD SA, PreviousSA;
	Agsi::ReadSFR(I2C_MASTER_ADDR(I2C_MASTER_O_SA), &SA, &PreviousSA, I2C_MASTER_SA_SA_MASK);
	SA >>= I2C_MASTER_SA_SA_SHIFT;

	if (RS) {
		trace("OOPS! Receive not implemented yet\n");
	} else {
		// CS is actually a status register; save status, modify local variable while
		// performing requested operations, then restore.
		DWORD CurrentCS, CS;
		Agsi::ReadSFR(I2C_MASTER_ADDR(I2C_MASTER_O_CS), &CurrentCS, &CS, (DWORD)-1);

		// Start bit
		Agsi::ReadSFR(I2C_MASTER_ADDR(I2C_MASTER_O_CS), &Current, &Previous, I2C_MASTER_CS_START);
		if (!Previous && Current) {
			CS &= ~I2C_MASTER_CS_IDLE;

			Queue.clear();

			// Clear raw interrupt flag
			Agsi::WriteSFR(I2C_MASTER_ADDR(I2C_MASTER_O_RIS), 0, I2C_MASTER_RIS_RIS);
		}

		// Run bit
		Agsi::ReadSFR(I2C_MASTER_ADDR(I2C_MASTER_O_CS), &Current, &Previous, I2C_MASTER_CS_RUN);
		if (!Previous && Current) {
			Agsi::ReadSFR(I2C_MASTER_ADDR(I2C_MASTER_O_DR), &Current, &Previous, 0xFF);
			Queue.push_back((unsigned char)Current);

			// Set raw interrupt flag (byte transfer completed)
			Agsi::WriteSFR(I2C_MASTER_ADDR(I2C_MASTER_O_RIS), I2C_MASTER_RIS_RIS, I2C_MASTER_RIS_RIS);
		}

		// Stop bit
		Agsi::ReadSFR(I2C_MASTER_ADDR(I2C_MASTER_O_CS), &Current, &Previous, I2C_MASTER_CS_STOP);
		if (!Previous && Current) {
			// Dispatch to slave handler
			Display::HandleTransfer(Queue);

			CS |= I2C_MASTER_CS_IDLE;
		}

		// Restore original value of SFR before write
		InWriteSFR = TRUE;
		Agsi::WriteSFR(I2C_MASTER_ADDR(I2C_MASTER_O_CS), CS, (DWORD)-1);
		InWriteSFR = FALSE;
	}
}

BOOL DefineAllWatches() {
	return
	Agsi::SetWatchOnSFR(I2C_MASTER_ADDR(I2C_MASTER_O_CS),	UpdateCS, AGSIWRITE);
}

BOOL DefineAllMenuEntries() {
	// The following line specifies the menu entry in the peripheral pull down menu. 
	// The '&' and the following character in the sting specifies the keyboard shortcut. <Alt+3> in this case.  
	//                    nDelim   *szText            *fp      nID   nDlgId         *pDlg;
	static AGSIMENU MenuI2CBegin =
		{  2,   "I2C &Master"        , NULL,  0,   0,      NULL};
	static AGSIMENU MenuI2CEnd =
		{  -2,   NULL        , NULL,  0,   0,      NULL};

	return
	Agsi::DefineMenuItem(&MenuI2CBegin) &&
	Display::DefineAllMenuEntries() &&
	Agsi::DefineMenuItem(&MenuI2CEnd);
}

BOOL Reset() {
	return
	Agsi::WriteSFR(I2C_MASTER_ADDR(I2C_MASTER_O_TPR), 0x00000001, (DWORD)-1);
}

};