/* Copyright (c) 2006, Catalin Patulea <cat@vv.carleton.ca> All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 * 
 *     * Neither the name of Catalin Patulea nor the names of its contributors may 
 *     be used to endorse or promote products derived from this software without 
 *     specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "Util.h"
#include "I2CMaster.h"
#include "Display.h"

BOOL APIENTRY DllMain(HINSTANCE hInstance, DWORD fdwReason, LPVOID lpReserved) {
	if (fdwReason == DLL_PROCESS_ATTACH) {
		DisableThreadLibraryCalls(_instance = hInstance);
	}

    return TRUE;
}

extern "C" DWORD AGSIEXPORT AgsiEntry(DWORD nCode, void *vp) {
	switch (nCode) {
	case AGSI_CHECK:
		// Experimentally, CPU type for Luminary LM3S811 is 7
		return *(DWORD *)vp == 7;
	case AGSI_INIT:
		trace("SSD0303 Initializing ----------------------------------\n");
		// Get AGSI function pointers
		if (!Agsi::Initialize(*(const AGSICONFIG *)vp)) {
			return FALSE;
		}

		return	I2CMaster::DefineAllSFR() &&
				I2CMaster::DefineAllVTREG() &&
				I2CMaster::DefineAllInterrupts() &&
				I2CMaster::DefineAllWatches() &&
				I2CMaster::DefineAllMenuEntries() &&
				Display::Init();
	case AGSI_RESET:
		return I2CMaster::Reset();
	case AGSI_TERMINATE:
		return Display::Terminate();
	case AGSI_PREPLL:
	case AGSI_POSTPLL:
		return TRUE;
	default:
		return FALSE;
	}
}