/* Copyright (c) 2006, Catalin Patulea <cat@vv.carleton.ca> All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 * 
 *     * Neither the name of Catalin Patulea nor the names of its contributors may 
 *     be used to endorse or promote products derived from this software without 
 *     specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "Util.h"

namespace Agsi {

// Function pointers
tAgsiDefineSFR             DefineSFR;
tAgsiDefineVTR             DefineVTR;
tAgsiSetWatchOnSFR         SetWatchOnSFR;
tAgsiSetWatchOnVTR         SetWatchOnVTR;
tAgsiSetWatchOnMemory      SetWatchOnMemory;
tAgsiCreateTimer           CreateTimer;
tAgsiDefineMenuItem        DefineMenuItem;
tAgsiWriteSFR              WriteSFR;
tAgsiReadSFR               ReadSFR;
tAgsiWriteVTR              WriteVTR;
tAgsiReadVTR               ReadVTR;
tAgsiSetSFRReadValue       SetSFRReadValue;
tAgsiReadMemory            ReadMemory;
tAgsiWriteMemory           WriteMemory;
tAgsiGetLastMemoryAddress  GetLastMemoryAddress;
tAgsiIsSimulatorAccess     IsSimulatorAccess;
tAgsiSetTimer              SetTimer;
tAgsiGetStates             GetStates;
tAgsiGetProgramCounter     GetProgramCounter;
tAgsiDeclareInterrupt      DeclareInterrupt;
tAgsiIsInInterrupt         IsInInterrupt;
tAgsiIsSleeping            IsSleeping;
tAgsiStopSimulator         StopSimulator;
tAgsiTriggerReset          TriggerReset;
tAgsiUpdateWindows         UpdateWindows;
tAgsiHandleFocus           HandleFocus;
tAgsiGetExternalClockRate  GetExternalClockRate;
tAgsiGetInternalClockRate  GetInternalClockRate;
tAgsiGetClockFactor        GetClockFactor;
tAgsiMessage               Message;
tAgsiGetTargetKey          GetTargetKey;
tAgsiSetTargetKey          SetTargetKey;
tAgsiGetSymbolByName       GetSymbolByName;
tAgsiGetSymbolByValue      GetSymbolByValue;
tAgsiExecuteCommand        ExecuteCommand;

BOOL Initialize(const AGSICONFIG& AgsiConfig) {
	DefineSFR            = (tAgsiDefineSFR)             GetProcAddress(AgsiConfig.m_hInstance, "AgsiDefineSFR");
	DefineVTR            = (tAgsiDefineVTR)             GetProcAddress(AgsiConfig.m_hInstance, "AgsiDefineVTR");
	DeclareInterrupt     = (tAgsiDeclareInterrupt)      GetProcAddress(AgsiConfig.m_hInstance, "AgsiDeclareInterrupt");
	SetWatchOnSFR        = (tAgsiSetWatchOnSFR)         GetProcAddress(AgsiConfig.m_hInstance, "AgsiSetWatchOnSFR");
	SetWatchOnVTR        = (tAgsiSetWatchOnVTR)         GetProcAddress(AgsiConfig.m_hInstance, "AgsiSetWatchOnVTR");
	SetWatchOnMemory     = (tAgsiSetWatchOnMemory)      GetProcAddress(AgsiConfig.m_hInstance, "AgsiSetWatchOnMemory");
	CreateTimer          = (tAgsiCreateTimer)           GetProcAddress(AgsiConfig.m_hInstance, "AgsiCreateTimer");
	DefineMenuItem       = (tAgsiDefineMenuItem)        GetProcAddress(AgsiConfig.m_hInstance, "AgsiDefineMenuItem");

	WriteSFR             = (tAgsiWriteSFR)              GetProcAddress(AgsiConfig.m_hInstance, "AgsiWriteSFR");
	ReadSFR              = (tAgsiReadSFR)               GetProcAddress(AgsiConfig.m_hInstance, "AgsiReadSFR");
	WriteVTR             = (tAgsiWriteVTR)              GetProcAddress(AgsiConfig.m_hInstance, "AgsiWriteVTR");
	ReadVTR              = (tAgsiReadVTR)               GetProcAddress(AgsiConfig.m_hInstance, "AgsiReadVTR");
	SetSFRReadValue      = (tAgsiSetSFRReadValue)       GetProcAddress(AgsiConfig.m_hInstance, "AgsiSetSFRReadValue");
	ReadMemory           = (tAgsiReadMemory)            GetProcAddress(AgsiConfig.m_hInstance, "AgsiReadMemory");
	WriteMemory          = (tAgsiWriteMemory)           GetProcAddress(AgsiConfig.m_hInstance, "AgsiWriteMemory");
	GetLastMemoryAddress = (tAgsiGetLastMemoryAddress)  GetProcAddress(AgsiConfig.m_hInstance, "AgsiGetLastMemoryAddress");
	IsSimulatorAccess    = (tAgsiIsSimulatorAccess)     GetProcAddress(AgsiConfig.m_hInstance, "AgsiIsSimulatorAccess");
	SetTimer             = (tAgsiSetTimer)              GetProcAddress(AgsiConfig.m_hInstance, "AgsiSetTimer");
	GetStates            = (tAgsiGetStates)             GetProcAddress(AgsiConfig.m_hInstance, "AgsiGetStates");
	GetProgramCounter    = (tAgsiGetProgramCounter)     GetProcAddress(AgsiConfig.m_hInstance, "AgsiGetProgramCounter");
	IsInInterrupt        = (tAgsiIsInInterrupt)         GetProcAddress(AgsiConfig.m_hInstance, "AgsiIsInInterrupt");
	IsSleeping           = (tAgsiIsSleeping)            GetProcAddress(AgsiConfig.m_hInstance, "AgsiIsSleeping");
	StopSimulator        = (tAgsiStopSimulator)         GetProcAddress(AgsiConfig.m_hInstance, "AgsiStopSimulator");
	TriggerReset         = (tAgsiTriggerReset)          GetProcAddress(AgsiConfig.m_hInstance, "AgsiTriggerReset");
	UpdateWindows        = (tAgsiUpdateWindows)         GetProcAddress(AgsiConfig.m_hInstance, "AgsiUpdateWindows");
	HandleFocus          = (tAgsiHandleFocus)           GetProcAddress(AgsiConfig.m_hInstance, "AgsiHandleFocus");
	GetExternalClockRate = (tAgsiGetExternalClockRate)  GetProcAddress(AgsiConfig.m_hInstance, "AgsiGetExternalClockRate");
	GetInternalClockRate = (tAgsiGetInternalClockRate)  GetProcAddress(AgsiConfig.m_hInstance, "AgsiGetInternalClockRate");
	GetClockFactor       = (tAgsiGetClockFactor)        GetProcAddress(AgsiConfig.m_hInstance, "AgsiGetClockFactor");
	Message              = (tAgsiMessage)               GetProcAddress(AgsiConfig.m_hInstance, "AgsiMessage");
	GetTargetKey         = (tAgsiGetTargetKey)          GetProcAddress(AgsiConfig.m_hInstance, "AgsiGetTargetKey");
	SetTargetKey         = (tAgsiSetTargetKey)          GetProcAddress(AgsiConfig.m_hInstance, "AgsiSetTargetKey");
	GetSymbolByName      = (tAgsiGetSymbolByName)       GetProcAddress(AgsiConfig.m_hInstance, "AgsiGetSymbolByName");
	GetSymbolByValue     = (tAgsiGetSymbolByValue)      GetProcAddress(AgsiConfig.m_hInstance, "AgsiGetSymbolByValue");
	ExecuteCommand       = (tAgsiExecuteCommand)        GetProcAddress(AgsiConfig.m_hInstance, "AgsiExecuteCommand");

	if ((DefineSFR            == NULL) ||
		(DefineVTR            == NULL) ||
		(DeclareInterrupt     == NULL) ||
		(SetWatchOnSFR        == NULL) ||
		(SetWatchOnVTR        == NULL) ||
		(SetWatchOnMemory     == NULL) ||
		(CreateTimer          == NULL) ||
		(DefineMenuItem       == NULL) ||

		(WriteSFR             == NULL) ||
		(ReadSFR              == NULL) ||
		(WriteVTR             == NULL) ||
		(ReadVTR              == NULL) ||
		(SetSFRReadValue      == NULL) ||
		(ReadMemory           == NULL) ||
		(WriteMemory          == NULL) ||
		(GetLastMemoryAddress == NULL) ||
		(IsSimulatorAccess    == NULL) ||
		(SetTimer             == NULL) ||
		(GetStates            == NULL) ||
		(GetProgramCounter    == NULL) ||
		(IsInInterrupt        == NULL) ||
		(IsSleeping           == NULL) ||
		(StopSimulator        == NULL) ||
		(TriggerReset         == NULL) ||
		(UpdateWindows        == NULL) ||
		(HandleFocus          == NULL) ||
		(GetExternalClockRate == NULL) ||
		(GetInternalClockRate == NULL) ||
		(GetClockFactor       == NULL) ||
		(Message              == NULL) ||
		(GetTargetKey         == NULL) ||
		(SetTargetKey         == NULL) ||
		(GetSymbolByName      == NULL) ||
		(GetSymbolByValue     == NULL) ||
		(ExecuteCommand       == NULL)) {
		return FALSE;
	} else {
		return TRUE;
	}
}

};

#ifdef _DEBUG
static HANDLE _out = INVALID_HANDLE_VALUE;
void trace(const char *fmt, ...) {
	if(_out == INVALID_HANDLE_VALUE) {
		AllocConsole();
		_out = GetStdHandle(STD_OUTPUT_HANDLE);
	}

	if(_out != INVALID_HANDLE_VALUE) {
		char buffer[1024];
		va_list args;
		DWORD written;

		va_start(args, fmt);
		wvsprintfA(buffer, fmt, args);
		va_end(args);

		WriteFile(_out, buffer, lstrlenA(buffer), &written, NULL);
	}
}
#endif

HINSTANCE _instance;