/* Copyright (c) 2006, Catalin Patulea <cat@vv.carleton.ca> All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 * 
 *     * Neither the name of Catalin Patulea nor the names of its contributors may 
 *     be used to endorse or promote products derived from this software without 
 *     specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UTIL_H
#define UTIL_H
#include "stdafx.h"

namespace Agsi {

// Initialize function pointers
BOOL Initialize(const AGSICONFIG& AgsiConfig);

// Callback functions that are ONLY allowed to call during the initialisation process
typedef BOOL      (*tAgsiDefineSFR) (const char* pszSfrName, AGSIADDR dwAddress, AGSITYPE eType, BYTE bBitPos);
typedef AGSIVTR   (*tAgsiDefineVTR) (const char* pszVtrName, AGSITYPE eType, DWORD dwValue);
typedef BOOL      (*tAgsiDeclareInterrupt)(AGSIINTERRUPT *pInterrupt);
typedef BOOL      (*tAgsiSetWatchOnSFR) (AGSIADDR SFRAddress, AGSICALLBACK pfnReadWrite, AGSIACCESS eAccess);
typedef BOOL      (*tAgsiSetWatchOnVTR) (AGSIVTR hVTR, AGSICALLBACK pfnReadWrite, AGSIACCESS eAccess);
typedef BOOL      (*tAgsiSetWatchOnMemory) (AGSIADDR StartAddress, AGSIADDR EndAddress, AGSICALLBACK pfnReadWrite, AGSIACCESS eAccess);
typedef AGSITIMER (*tAgsiCreateTimer) (AGSICALLBACK pfnTimer);
typedef BOOL      (*tAgsiDefineMenuItem)(AGSIMENU *pDym);

// Callback functions that are NOT allowed to call during the initialisation process
typedef BOOL      (*tAgsiWriteSFR) (AGSIADDR SFRAddress, DWORD dwValue, DWORD dwMask);
typedef BOOL      (*tAgsiReadSFR)  (AGSIADDR SFRAddress, DWORD* pdwCurrentValue, DWORD* pdwPreviousValue, DWORD dwMask);
typedef BOOL      (*tAgsiWriteVTR) (AGSIVTR hVTR, DWORD dwValue);
typedef BOOL      (*tAgsiReadVTR) (AGSIVTR hVTR, DWORD* pdwCurrentValue);
typedef BOOL      (*tAgsiSetSFRReadValue)(DWORD dwValue);
typedef BOOL      (*tAgsiReadMemory) (AGSIADDR Address, DWORD dwCount, BYTE* pbValue);
typedef BOOL      (*tAgsiWriteMemory)(AGSIADDR Address, DWORD dwCount, BYTE* pbValue);
typedef AGSIADDR  (*tAgsiGetLastMemoryAddress)(void);
typedef BOOL      (*tAgsiIsSimulatorAccess)(void);
typedef BOOL      (*tAgsiSetTimer)(AGSITIMER hTimer, DWORD dwClock);
typedef UINT64    (*tAgsiGetStates)(void);
typedef AGSIADDR  (*tAgsiGetProgramCounter)(void);
typedef DWORD     (*tAgsiIsInInterrupt)(void);
typedef BOOL      (*tAgsiIsSleeping)(void);
typedef void      (*tAgsiStopSimulator)(void);
typedef void      (*tAgsiTriggerReset)(void);
typedef void      (*tAgsiUpdateWindows)(void);
typedef void      (*tAgsiHandleFocus) (HWND hwndDialog);
typedef DWORD     (*tAgsiGetExternalClockRate)(void);
typedef DWORD     (*tAgsiGetInternalClockRate)(void);
typedef double    (*tAgsiGetClockFactor)(void);
typedef void      (*tAgsiMessage)(const char* pszFormat, ...);
typedef const char*(*tAgsiGetTargetKey)(const char* pszKey);
typedef BOOL      (*tAgsiSetTargetKey)(const char* pszKey, const char *pszString);
typedef DWORD     (*tAgsiGetSymbolByName) (AGSISYMDSC *vp);
typedef DWORD     (*tAgsiGetSymbolByValue)(AGSISYMDSC *vp);
typedef void      (*tAgsiExecuteCommand)(const char *pszCommand);

// Function pointers
extern tAgsiDefineSFR             DefineSFR;
extern tAgsiDefineVTR             DefineVTR;
extern tAgsiSetWatchOnSFR         SetWatchOnSFR;
extern tAgsiSetWatchOnVTR         SetWatchOnVTR;
extern tAgsiSetWatchOnMemory      SetWatchOnMemory;
extern tAgsiCreateTimer           CreateTimer;
extern tAgsiDefineMenuItem        DefineMenuItem;
extern tAgsiWriteSFR              WriteSFR;
extern tAgsiReadSFR               ReadSFR;
extern tAgsiWriteVTR              WriteVTR;
extern tAgsiReadVTR               ReadVTR;
extern tAgsiSetSFRReadValue       SetSFRReadValue;
extern tAgsiReadMemory            ReadMemory;
extern tAgsiWriteMemory           WriteMemory;
extern tAgsiGetLastMemoryAddress  GetLastMemoryAddress;
extern tAgsiIsSimulatorAccess     IsSimulatorAccess;
extern tAgsiSetTimer              SetTimer;
extern tAgsiGetStates             GetStates;
extern tAgsiGetProgramCounter     GetProgramCounter;
extern tAgsiDeclareInterrupt      DeclareInterrupt;
extern tAgsiIsInInterrupt         IsInInterrupt;
extern tAgsiIsSleeping            IsSleeping;
extern tAgsiStopSimulator         StopSimulator;
extern tAgsiTriggerReset          TriggerReset;
extern tAgsiUpdateWindows         UpdateWindows;
extern tAgsiHandleFocus           HandleFocus;
extern tAgsiGetExternalClockRate  GetExternalClockRate;
extern tAgsiGetInternalClockRate  GetInternalClockRate;
extern tAgsiGetClockFactor        GetClockFactor;
extern tAgsiMessage               Message;
extern tAgsiGetTargetKey          GetTargetKey;
extern tAgsiSetTargetKey          SetTargetKey;
extern tAgsiGetSymbolByName       GetSymbolByName;
extern tAgsiGetSymbolByValue      GetSymbolByValue;
extern tAgsiExecuteCommand        ExecuteCommand;

};

#ifdef _DEBUG
// If debug, make it an actual function call
void trace(const char *fmt, ...);
#else
// In release, force-inline it to nothing at all
void __forceinline trace(const char *fmt, ...) {
}
#endif

// Keeps track of our instance handler
extern HINSTANCE _instance;

#endif